@@ -2,7 +2,7 @@
 
 import numpy as np
 import matplotlib.pyplot as plt
-from openbox import Optimizer, sp
+from openbox import Optimizer, space as sp
 
 
 # objective function
@@ -17,7 +17,7 @@ def BraninCurrin(config: sp.Configuration):
          / (100 * x1 ** 3 + 500 * x1 ** 2 + 4 * x1 + 20)
 
     result = dict()
-    result['objs'] = [f1, f2]
+    result['objectives'] = [f1, f2]
     return result
 
 
@@ -25,7 +25,7 @@ if __name__ == "__main__":
     # search space
     space = sp.Space()
     x1 = sp.Real("x1", 0, 1)
-    x2 = sp.Real("x2", 0, 1)
+    x2 = sp.Real("x2", 1e-6, 1)
     space.add_variables([x1, x2])
 
     # provide reference point if using EHVI method
@@ -35,7 +35,7 @@ if __name__ == "__main__":
     opt = Optimizer(
         BraninCurrin,
         space,
-        num_objs=2,
+        num_objectives=2,
         num_constraints=0,
         max_runs=50,
         surrogate_type='gp',
@@ -48,20 +48,14 @@ if __name__ == "__main__":
         task_id='mo',
         random_state=1,
     )
-    opt.run()
+    history = opt.run()
+    print(history)
 
     # plot pareto front
-    pareto_front = np.asarray(opt.get_history().get_pareto_front())
-    if pareto_front.shape[-1] in (2, 3):
-        if pareto_front.shape[-1] == 2:
-            plt.scatter(pareto_front[:, 0], pareto_front[:, 1])
-            plt.xlabel('Objective 1')
-            plt.ylabel('Objective 2')
-        elif pareto_front.shape[-1] == 3:
-            ax = plt.axes(projection='3d')
-            ax.scatter3D(pareto_front[:, 0], pareto_front[:, 1], pareto_front[:, 2])
-            ax.set_xlabel('Objective 1')
-            ax.set_ylabel('Objective 2')
-            ax.set_zlabel('Objective 3')
-        plt.title('Pareto Front')
+    if history.num_objectives in [2, 3]:
+        history.plot_pareto_front()  # support 2 or 3 objectives
         plt.show()
+
+    # plot hypervolume (optimal hypervolume of BraninCurrin is approximated using NSGA-II)
+    history.plot_hypervolumes(optimal_hypervolume=59.36011874867746, logy=True)
+    plt.show()
