@@ -2,7 +2,7 @@
 
 import numpy as np
 import matplotlib.pyplot as plt
-from openbox import Optimizer, sp
+from openbox import Optimizer, space as sp
 
 
 # objective function
@@ -16,7 +16,7 @@ def CONSTR(config: sp.Configuration):
     c2 = 1.0 - 9.0 * x1 + x2
 
     result = dict()
-    result['objs'] = [obj1, obj2]
+    result['objectives'] = [obj1, obj2]
     result['constraints'] = [c1, c2]
     return result
 
@@ -35,7 +35,7 @@ if __name__ == "__main__":
     opt = Optimizer(
         CONSTR,
         space,
-        num_objs=2,
+        num_objectives=2,
         num_constraints=2,
         max_runs=20,
         surrogate_type='gp',
@@ -48,20 +48,14 @@ if __name__ == "__main__":
         task_id='moc',
         random_state=1,
     )
-    opt.run()
+    history = opt.run()
+    print(history)
 
     # plot pareto front
-    pareto_front = np.asarray(opt.get_history().get_pareto_front())
-    if pareto_front.shape[-1] in (2, 3):
-        if pareto_front.shape[-1] == 2:
-            plt.scatter(pareto_front[:, 0], pareto_front[:, 1])
-            plt.xlabel('Objective 1')
-            plt.ylabel('Objective 2')
-        elif pareto_front.shape[-1] == 3:
-            ax = plt.axes(projection='3d')
-            ax.scatter3D(pareto_front[:, 0], pareto_front[:, 1], pareto_front[:, 2])
-            ax.set_xlabel('Objective 1')
-            ax.set_ylabel('Objective 2')
-            ax.set_zlabel('Objective 3')
-        plt.title('Pareto Front')
+    if history.num_objectives in [2, 3]:
+        history.plot_pareto_front()  # support 2 or 3 objectives
         plt.show()
+
+    # plot hypervolume (optimal hypervolume of CONSTR is approximated using NSGA-II)
+    history.plot_hypervolumes(optimal_hypervolume=92.02004226679216, logy=True)
+    plt.show()
