@@ -2,7 +2,7 @@
 
 import numpy as np
 import matplotlib.pyplot as plt
-from openbox import Optimizer, sp
+from openbox import Optimizer, space as sp
 
 
 # Define Search Space
@@ -17,7 +17,7 @@ def branin(config):
     x1, x2 = config['x1'], config['x2']
     y = (x2 - 5.1 / (4 * np.pi ** 2) * x1 ** 2 + 5 / np.pi * x1 - 6) ** 2 \
         + 10 * (1 - 1 / (8 * np.pi)) * np.cos(x1) + 10
-    return {'objs': (y,)}
+    return {'objectives': [y]}
 
 
 # Run
@@ -41,4 +41,4 @@ if __name__ == "__main__":
     # install pyrfr to use get_importance()
     # print(history.get_importance())
 
-    # history.visualize_jupyter()
+    # history.visualize_hiplot()
