@@ -1,11 +1,13 @@
-# This en code is licensed under the MIT license found in the
-# LICENSE file in the root directory of this en tree.
+# This source code is licensed under the MIT license found in the
+# LICENSE file in the root directory of this source tree.
 
-from .pkginfo import version as __version__, package_name
+from .utils import color_logger as logger
 
 from .utils import space
 from .utils import space as sp
 
+from .utils.history import Observation, History
+
 from .optimizer.generic_smbo import SMBO as Optimizer
 from .optimizer.parallel_smbo import pSMBO as ParallelOptimizer
 from .optimizer.message_queue_smbo import mqSMBO as DistributedOptimizer
@@ -22,22 +24,25 @@ from .core.tpe_advisor import TPE_Advisor
 from .core.ea_advisor import EA_Advisor
 from .core.mc_advisor import MCAdvisor
 
-from .core.base import Observation
-
 from .utils.tuning import get_config_space, get_objective_function
 
 from .utils.test_install import run_test
 
+
+__version__ = version = "0.8.0"
+
+
 __all__ = [
-    "__version__", "__package__", "package_name",
+    "__version__", "version",
+    "logger",
     "sp", "space",
+    "Observation", "History",
     "Optimizer", "ParallelOptimizer", "DistributedOptimizer", "DistributedWorker",
     "NSGAOptimizer",
     "create_optimizer",
     "Advisor",
     "AsyncBatchAdvisor", "SyncBatchAdvisor",
     "RandomAdvisor", "TPE_Advisor", "EA_Advisor", "MCAdvisor",
-    "Observation",
     "get_config_space", "get_objective_function",
     "run_test",
 ]
