@@ -6,7 +6,6 @@
 # Author: Aaron Klein, Marius Lindauer
 
 import abc
-import logging
 import time
 import warnings
 from typing import Iterable, List, Union, Tuple, Optional
@@ -14,11 +13,12 @@ import random
 import scipy.optimize
 import numpy as np
 
+from openbox import logger
 from openbox.acquisition_function.acquisition import AbstractAcquisitionFunction
 from openbox.utils.config_space import get_one_exchange_neighbourhood, \
     Configuration, ConfigurationSpace
 from openbox.acq_maximizer.random_configuration_chooser import ChooserNoCoolDown, ChooserProb
-from openbox.utils.history_container import HistoryContainer, MultiStartHistoryContainer
+from openbox.utils.history import History, MultiStartHistory
 from openbox.utils.util_funcs import get_types
 from openbox.utils.constants import MAXINT
 
@@ -31,9 +31,9 @@ class AcquisitionFunctionMaximizer(object, metaclass=abc.ABCMeta):
 
     Parameters
     ----------
-    acquisition_function : ~openbox.acquisition_function.acquisition.AbstractAcquisitionFunction
+    acquisition_function : AbstractAcquisitionFunction
 
-    config_space : ~openbox.config_space.ConfigurationSpace
+    config_space : ConfigurationSpace
 
     rng : np.random.RandomState or int, optional
     """
@@ -44,21 +44,18 @@ class AcquisitionFunctionMaximizer(object, metaclass=abc.ABCMeta):
             config_space: ConfigurationSpace,
             rng: Union[bool, np.random.RandomState] = None
     ):
-        self.logger = logging.getLogger(
-            self.__module__ + "." + self.__class__.__name__
-        )
         self.acquisition_function = acquisition_function
         self.config_space = config_space
 
         if rng is None:
-            self.logger.debug('no rng given, using default seed of 1')
+            logger.debug('no rng given, using default seed of 1')
             self.rng = np.random.RandomState(seed=1)
         else:
             self.rng = rng
 
     def maximize(
             self,
-            runhistory: HistoryContainer,
+            runhistory: History,
             num_points: int,
             **kwargs
     ) -> Iterable[Configuration]:
@@ -66,10 +63,8 @@ class AcquisitionFunctionMaximizer(object, metaclass=abc.ABCMeta):
 
         Parameters
         ----------
-        runhistory: ~openbox.utils.history_container.HistoryContainer
+        runhistory: openbox.utils.history.History
             runhistory object
-        stats: ~openbox.stats.stats.Stats
-            current stats object
         num_points: int
             number of points to be sampled
         **kwargs
@@ -84,7 +79,7 @@ class AcquisitionFunctionMaximizer(object, metaclass=abc.ABCMeta):
     @abc.abstractmethod
     def _maximize(
             self,
-            runhistory: HistoryContainer,
+            runhistory: History,
             num_points: int,
             **kwargs
     ) -> Iterable[Tuple[float, Configuration]]:
@@ -96,10 +91,8 @@ class AcquisitionFunctionMaximizer(object, metaclass=abc.ABCMeta):
 
         Parameters
         ----------
-        runhistory: ~openbox.utils.history_container.HistoryContainer
+        runhistory: openbox.utils.history.History
             runhistory object
-        stats: ~openbox.stats.stats.Stats
-            current stats object
         num_points: int
             number of points to be sampled
         **kwargs
@@ -154,7 +147,7 @@ class CMAESOptimizer(AcquisitionFunctionMaximizer):
 
     def _maximize(
             self,
-            runhistory: HistoryContainer,
+            runhistory: History,
             num_points: int,
             **kwargs
     ) -> Iterable[Tuple[float, Configuration]]:
@@ -162,7 +155,7 @@ class CMAESOptimizer(AcquisitionFunctionMaximizer):
 
     def maximize(
             self,
-            runhistory: HistoryContainer,
+            runhistory: History,
             num_points: int,
             **kwargs
     ) -> Iterable[Tuple[float, Configuration]]:
@@ -215,9 +208,9 @@ class LocalSearch(AcquisitionFunctionMaximizer):
 
     Parameters
     ----------
-    acquisition_function : ~openbox.acquisition_function.acquisition.AbstractAcquisitionFunction
+    acquisition_function : AbstractAcquisitionFunction
 
-    config_space : ~openbox.config_space.ConfigurationSpace
+    config_space : ConfigurationSpace
 
     rng : np.random.RandomState or int, optional
 
@@ -243,7 +236,7 @@ class LocalSearch(AcquisitionFunctionMaximizer):
 
     def _maximize(
             self,
-            runhistory: HistoryContainer,
+            runhistory: History,
             num_points: int,
             **kwargs
     ) -> List[Tuple[float, Configuration]]:
@@ -253,13 +246,11 @@ class LocalSearch(AcquisitionFunctionMaximizer):
 
         Parameters
         ----------
-        runhistory: ~openbox.utils.history_container.HistoryContainer
+        runhistory: openbox.utils.history.History
             runhistory object
-        stats: ~openbox.stats.stats.Stats
-            current stats object
         num_points: int
             number of points to be sampled
-        ***kwargs:
+        **kwargs:
             Additional parameters that will be passed to the
             acquisition function
 
@@ -299,7 +290,7 @@ class LocalSearch(AcquisitionFunctionMaximizer):
                 size=num_points)
         else:
             # initiate local search with best configurations from previous runs
-            configs_previous_runs = runhistory.get_all_configs()
+            configs_previous_runs = runhistory.configurations
             configs_previous_runs_sorted = self._sort_configs_by_acq_value(
                 configs_previous_runs)
             num_configs_local_search = int(min(
@@ -330,7 +321,7 @@ class LocalSearch(AcquisitionFunctionMaximizer):
 
             local_search_steps += 1
             if local_search_steps % 1000 == 0:
-                self.logger.warning(
+                logger.warning(
                     "Local search took already %d iterations. Is it maybe "
                     "stuck in a infinite loop?", local_search_steps
                 )
@@ -351,7 +342,7 @@ class LocalSearch(AcquisitionFunctionMaximizer):
                 time_n.append(time.time() - s_time)
 
                 if acq_val > acq_val_incumbent:
-                    self.logger.debug("Switch to one of the neighbors")
+                    # logger.debug("Switch to one of the neighbors")
                     incumbent = neighbor
                     acq_val_incumbent = acq_val
                     changed_inc = True
@@ -362,7 +353,7 @@ class LocalSearch(AcquisitionFunctionMaximizer):
                      local_search_steps == self.max_steps):
                 if len(time_n) == 0:
                     time_n.append(0.0)
-                self.logger.debug("Local search took %d steps and looked at %d "
+                logger.debug("Local search took %d steps and looked at %d "
                                   "configurations. Computing the acquisition "
                                   "value for one configuration took %f seconds"
                                   " on average.",
@@ -378,16 +369,16 @@ class RandomSearch(AcquisitionFunctionMaximizer):
 
     Parameters
     ----------
-    acquisition_function : ~openbox.acquisition_function.acquisition.AbstractAcquisitionFunction
+    acquisition_function : AbstractAcquisitionFunction
 
-    config_space : ~openbox.config_space.ConfigurationSpace
+    config_space : ConfigurationSpace
 
     rng : np.random.RandomState or int, optional
     """
 
     def _maximize(
             self,
-            runhistory: HistoryContainer,
+            runhistory: History,
             num_points: int,
             _sorted: bool = False,
             **kwargs
@@ -396,7 +387,7 @@ class RandomSearch(AcquisitionFunctionMaximizer):
 
         Parameters
         ----------
-        runhistory: ~openbox.utils.history_container.HistoryContainer
+        runhistory: openbox.utils.history.History
             runhistory object
         num_points: int
             number of points to be sampled
@@ -437,9 +428,9 @@ class InterleavedLocalAndRandomSearch(AcquisitionFunctionMaximizer):
 
     Parameters
     ----------
-    acquisition_function : ~openbox.acquisition_function.acquisition.AbstractAcquisitionFunction
+    acquisition_function : AbstractAcquisitionFunction
 
-    config_space : ~openbox.config_space.ConfigurationSpace
+    config_space : ConfigurationSpace
 
     rng : np.random.RandomState or int, optional
 
@@ -490,7 +481,7 @@ class InterleavedLocalAndRandomSearch(AcquisitionFunctionMaximizer):
 
     def maximize(
             self,
-            runhistory: HistoryContainer,
+            runhistory: History,
             num_points: int,
             random_configuration_chooser=None,
             **kwargs
@@ -499,7 +490,7 @@ class InterleavedLocalAndRandomSearch(AcquisitionFunctionMaximizer):
 
         Parameters
         ----------
-        runhistory: ~openbox.utils.history_container.HistoryContainer
+        runhistory: openbox.utils.history.History
             runhistory object
         num_points: int
             number of points to be sampled
@@ -540,7 +531,7 @@ class InterleavedLocalAndRandomSearch(AcquisitionFunctionMaximizer):
                 + next_configs_by_local_search
         )
         next_configs_by_acq_value.sort(reverse=True, key=lambda x: x[0])
-        self.logger.debug(
+        logger.debug(
             "First 10 acq func (origin) values of selected configurations: %s",
             str([[_[0], _[1].origin] for _ in next_configs_by_acq_value[:10]])
         )
@@ -554,7 +545,7 @@ class InterleavedLocalAndRandomSearch(AcquisitionFunctionMaximizer):
 
     def _maximize(
             self,
-            runhistory: HistoryContainer,
+            runhistory: History,
             num_points: int,
             **kwargs
     ) -> Iterable[Tuple[float, Configuration]]:
@@ -567,9 +558,9 @@ class ScipyOptimizer(AcquisitionFunctionMaximizer):
 
     Parameters
     ----------
-    acquisition_function : ~openbox.acquisition_function.acquisition.AbstractAcquisitionFunction
+    acquisition_function : AbstractAcquisitionFunction
 
-    config_space : ~openbox.config_space.ConfigurationSpace
+    config_space : ConfigurationSpace
 
     rng : np.random.RandomState or int, optional
     """
@@ -593,7 +584,7 @@ class ScipyOptimizer(AcquisitionFunctionMaximizer):
 
     def maximize(
             self,
-            runhistory: HistoryContainer,
+            runhistory: History,
             initial_config=None,
             **kwargs
     ) -> List[Tuple[float, Configuration]]:
@@ -621,7 +612,7 @@ class ScipyOptimizer(AcquisitionFunctionMaximizer):
                                              bounds=self.bounds,
                                              **self.scipy_config)
         if not result.success:
-            self.logger.debug('Scipy optimizer failed. Info:\n%s' % (result,))
+            logger.debug('Scipy optimizer failed. Info:\n%s' % (result,))
         try:
             x = np.clip(result.x, 0.0, 1.0)  # fix numerical problem in L-BFGS-B
             config = Configuration(self.config_space, vector=x)
@@ -632,7 +623,7 @@ class ScipyOptimizer(AcquisitionFunctionMaximizer):
             pass
 
         if not acq_configs:  # empty
-            self.logger.warning('Scipy optimizer failed. Return empty config list. Info:\n%s' % (result,))
+            logger.warning('Scipy optimizer failed. Return empty config list. Info:\n%s' % (result,))
 
         challengers = ChallengerList([config for _, config in acq_configs],
                                      self.config_space,
@@ -642,7 +633,7 @@ class ScipyOptimizer(AcquisitionFunctionMaximizer):
 
     def _maximize(
             self,
-            runhistory: HistoryContainer,
+            runhistory: History,
             num_points: int,
             **kwargs
     ) -> Iterable[Tuple[float, Configuration]]:
@@ -655,9 +646,9 @@ class RandomScipyOptimizer(AcquisitionFunctionMaximizer):
 
     Parameters
     ----------
-    acquisition_function : ~openbox.acquisition_function.acquisition.AbstractAcquisitionFunction
+    acquisition_function : AbstractAcquisitionFunction
 
-    config_space : ~openbox.config_space.ConfigurationSpace
+    config_space : ConfigurationSpace
 
     rng : np.random.RandomState or int, optional
     """
@@ -686,7 +677,7 @@ class RandomScipyOptimizer(AcquisitionFunctionMaximizer):
 
     def maximize(
             self,
-            runhistory: HistoryContainer,
+            runhistory: History,
             num_points: int,
             num_trials=10,
             **kwargs
@@ -708,7 +699,7 @@ class RandomScipyOptimizer(AcquisitionFunctionMaximizer):
             acq_configs.extend(zip(scipy_acqs, scipy_configs))
             success_count += 1
         if success_count == 0:
-            self.logger.warning('None of Scipy optimizations are successful in RandomScipyOptimizer.')
+            logger.warning('None of Scipy optimizations are successful in RandomScipyOptimizer.')
 
         # shuffle for random tie-break
         self.rng.shuffle(acq_configs)
@@ -726,7 +717,7 @@ class RandomScipyOptimizer(AcquisitionFunctionMaximizer):
 
     def _maximize(
             self,
-            runhistory: HistoryContainer,
+            runhistory: History,
             num_points: int,
             **kwargs
     ) -> Iterable[Tuple[float, Configuration]]:
@@ -739,9 +730,9 @@ class ScipyGlobalOptimizer(AcquisitionFunctionMaximizer):
 
     Parameters
     ----------
-    acquisition_function : ~openbox.acquisition_function.acquisition.AbstractAcquisitionFunction
+    acquisition_function : AbstractAcquisitionFunction
 
-    config_space : ~openbox.config_space.ConfigurationSpace
+    config_space : ConfigurationSpace
 
     rng : np.random.RandomState or int, optional
     """
@@ -762,7 +753,7 @@ class ScipyGlobalOptimizer(AcquisitionFunctionMaximizer):
 
     def maximize(
             self,
-            runhistory: HistoryContainer,
+            runhistory: History,
             initial_config=None,
             **kwargs
     ) -> List[Tuple[float, Configuration]]:
@@ -775,7 +766,7 @@ class ScipyGlobalOptimizer(AcquisitionFunctionMaximizer):
         result = scipy.optimize.differential_evolution(func=negative_acquisition,
                                                        bounds=self.bounds)
         if not result.success:
-            self.logger.debug('Scipy differential evolution optimizer failed. Info:\n%s' % (result,))
+            logger.debug('Scipy differential evolution optimizer failed. Info:\n%s' % (result,))
         try:
             config = Configuration(self.config_space, vector=result.x)
             acq = self.acquisition_function(result.x, convert=False)
@@ -784,7 +775,7 @@ class ScipyGlobalOptimizer(AcquisitionFunctionMaximizer):
             pass
 
         if not acq_configs:  # empty
-            self.logger.warning('Scipy differential evolution optimizer failed. Return empty config list. Info:\n%s' % (result,))
+            logger.warning('Scipy differential evolution optimizer failed. Return empty config list. Info:\n%s' % (result,))
 
         challengers = ChallengerList([config for _, config in acq_configs],
                                      self.config_space,
@@ -794,7 +785,7 @@ class ScipyGlobalOptimizer(AcquisitionFunctionMaximizer):
 
     def _maximize(
             self,
-            runhistory: HistoryContainer,
+            runhistory: History,
             num_points: int,
             **kwargs
     ) -> Iterable[Tuple[float, Configuration]]:
@@ -807,9 +798,9 @@ class StagedBatchScipyOptimizer(AcquisitionFunctionMaximizer):
 
     Parameters
     ----------
-    acquisition_function : ~openbox.acquisition_function.acquisition.AbstractAcquisitionFunction
+    acquisition_function : AbstractAcquisitionFunction
 
-    config_space : ~openbox.config_space.ConfigurationSpace
+    config_space : ConfigurationSpace
 
     num_random : Number of random chosen points
 
@@ -882,7 +873,7 @@ class StagedBatchScipyOptimizer(AcquisitionFunctionMaximizer):
 
         # return result.x even failed. may because 'STOP: TOTAL NO. of ITERATIONS REACHED LIMIT'
         # if not result.success:
-        #     self.logger.warning('Scipy minimizer %s failed in this round: %s.' % (self.method, result))
+        #     logger.warning('Scipy minimizer %s failed in this round: %s.' % (self.method, result))
         #     return None
 
         #print(result.x.reshape(shapeX))    # todo remove
@@ -890,7 +881,7 @@ class StagedBatchScipyOptimizer(AcquisitionFunctionMaximizer):
 
     def maximize(
             self,
-            runhistory: HistoryContainer,
+            runhistory: History,
             num_points: int,  # todo useless
             **kwargs
     ) -> List[Tuple[float, Configuration]]:
@@ -949,7 +940,7 @@ class StagedBatchScipyOptimizer(AcquisitionFunctionMaximizer):
 
     def _maximize(
             self,
-            runhistory: HistoryContainer,
+            runhistory: History,
             num_points: int,
             **kwargs
     ) -> Iterable[Tuple[float, Configuration]]:
@@ -961,9 +952,9 @@ class MESMO_Optimizer(AcquisitionFunctionMaximizer):
 
     Parameters
     ----------
-    acquisition_function : ~openbox.acquisition_function.acquisition.AbstractAcquisitionFunction
+    acquisition_function : AbstractAcquisitionFunction
 
-    config_space : ~openbox.config_space.ConfigurationSpace
+    config_space : ConfigurationSpace
 
     rng : np.random.RandomState or int, optional
 
@@ -986,7 +977,7 @@ class MESMO_Optimizer(AcquisitionFunctionMaximizer):
 
     def maximize(
             self,
-            runhistory: HistoryContainer,
+            runhistory: History,
             num_points: int,  # todo useless
             **kwargs
     ) -> Iterable[Configuration]:
@@ -994,7 +985,7 @@ class MESMO_Optimizer(AcquisitionFunctionMaximizer):
 
         Parameters
         ----------
-        runhistory: ~openbox.utils.history_container.HistoryContainer
+        runhistory: openbox.utils.history.History
             runhistory object
         num_points: int
             number of points to be sampled
@@ -1054,7 +1045,7 @@ class MESMO_Optimizer(AcquisitionFunctionMaximizer):
 
     def _maximize(
             self,
-            runhistory: HistoryContainer,
+            runhistory: History,
             num_points: int,
             **kwargs
     ) -> Iterable[Tuple[float, Configuration]]:
@@ -1066,9 +1057,9 @@ class USeMO_Optimizer(AcquisitionFunctionMaximizer):
 
     Parameters
     ----------
-    acquisition_function : ~openbox.acquisition_function.acquisition.AbstractAcquisitionFunction
+    acquisition_function : AbstractAcquisitionFunction
 
-    config_space : ~openbox.config_space.ConfigurationSpace
+    config_space : ConfigurationSpace
 
     rng : np.random.RandomState or int, optional
 
@@ -1086,7 +1077,7 @@ class USeMO_Optimizer(AcquisitionFunctionMaximizer):
 
     def maximize(
             self,
-            runhistory: HistoryContainer,
+            runhistory: History,
             num_points: int,  # useless in USeMO
             **kwargs
     ) -> Iterable[Configuration]:
@@ -1094,7 +1085,7 @@ class USeMO_Optimizer(AcquisitionFunctionMaximizer):
 
         Parameters
         ----------
-        runhistory: ~openbox.utils.history_container.HistoryContainer
+        runhistory: openbox.utils.history.History
             runhistory object
         num_points: int
             number of points to be sampled
@@ -1134,7 +1125,7 @@ class USeMO_Optimizer(AcquisitionFunctionMaximizer):
 
     def _maximize(
             self,
-            runhistory: HistoryContainer,
+            runhistory: History,
             num_points: int,
             **kwargs
     ) -> Iterable[Tuple[float, Configuration]]:
@@ -1162,7 +1153,7 @@ class batchMCOptimizer(AcquisitionFunctionMaximizer):
 
     def maximize(
             self,
-            runhistory: Union[HistoryContainer, MultiStartHistoryContainer],
+            runhistory: Union[History, MultiStartHistory],
             num_points: int,
             _sorted: bool = True,
             **kwargs
@@ -1171,7 +1162,7 @@ class batchMCOptimizer(AcquisitionFunctionMaximizer):
 
         Parameters
         ----------
-        runhistory: ~openbox.utils.history_container.HistoryContainer
+        runhistory: openbox.utils.history.History
             runhistory object
         num_points: int
             number of points to be sampled
@@ -1200,12 +1191,13 @@ class batchMCOptimizer(AcquisitionFunctionMaximizer):
                 lower_bounds = None
                 upper_bounds = None
             else:
-                assert isinstance(runhistory, MultiStartHistoryContainer)
-                if runhistory.num_objs > 1:
+                assert isinstance(runhistory, MultiStartHistory)
+                if runhistory.num_objectives > 1:
                     # TODO implement adaptive strategy to choose trust region center for MO
                     raise NotImplementedError()
                 else:
-                    x_center = random.choice(runhistory.get_incumbents())[0].get_array()
+                    incumbent_config = self.rng.choice(runhistory.get_incumbent_configs())
+                    x_center = incumbent_config.get_array()
                     lower_bounds = x_center - turbo_state.length / 2.0
                     upper_bounds = x_center + turbo_state.length / 2.0
 
@@ -1228,7 +1220,7 @@ class batchMCOptimizer(AcquisitionFunctionMaximizer):
 
     def _maximize(
             self,
-            runhistory: HistoryContainer,
+            runhistory: History,
             num_points: int,
             **kwargs
     ) -> Iterable[Tuple[float, Configuration]]:
