@@ -1,9 +1,9 @@
 # License: MIT
 
 from math import ceil
+from openbox import logger
 from openbox.apps.multi_fidelity.utils import sample_configuration
 from openbox.apps.multi_fidelity.async_mq_sh_v0 import async_mqSuccessiveHalving_v0
-
 from openbox.utils.config_space import ConfigurationSpace
 
 
@@ -46,7 +46,7 @@ class async_mqHyperband_v0(async_mqSuccessiveHalving_v0):
             # construct iteration list
             self.hb_bracket_list.append([r] * n)
         self.hb_iter_list = self.hb_bracket_list[0]
-        self.logger.info('hyperband iteration lists of all brackets: %s. init bracket: %s.'
+        logger.info('hyperband iteration lists of all brackets: %s. init bracket: %s.'
                          % (self.hb_bracket_list, self.hb_iter_list))
 
     def choose_next(self):
@@ -70,5 +70,5 @@ class async_mqHyperband_v0(async_mqSuccessiveHalving_v0):
             if self.hb_bracket_id == len(self.hb_bracket_list):
                 self.hb_bracket_id = 0
             self.hb_iter_list = self.hb_bracket_list[self.hb_bracket_id]
-            self.logger.info('iteration list of next bracket: %s' % self.hb_iter_list)
+            logger.info('iteration list of next bracket: %s' % self.hb_iter_list)
         return next_n_iteration
