@@ -7,6 +7,7 @@ from math import log, ceil
 from sklearn.model_selection import KFold
 from scipy.optimize import minimize
 
+from openbox import logger
 from openbox.apps.multi_fidelity.async_mq_hb import async_mqHyperband
 from openbox.apps.multi_fidelity.utils import RUNNING, COMPLETED, PROMOTED
 from openbox.apps.multi_fidelity.utils import sample_configuration
@@ -20,7 +21,7 @@ from openbox.surrogate.base.rf_with_instances import RandomForestWithInstances
 from openbox.acq_maximizer.ei_optimization import InterleavedLocalAndRandomSearch, RandomSearch
 from openbox.acq_maximizer.random_configuration_chooser import ChooserProb
 from openbox.utils.config_space.util import convert_configurations_to_array
-from openbox.utils.history_container import HistoryContainer
+from openbox.utils.history import History, Observation
 
 
 class async_mqMFES(async_mqHyperband):
@@ -67,7 +68,7 @@ class async_mqMFES(async_mqHyperband):
         if init_weight is None:
             init_weight = [1. / self.s_max] * self.s_max + [0.]
             assert len(init_weight) == (self.s_max + 1)
-        self.logger.info("Initialize weight to %s" % init_weight[:self.s_max + 1])
+        logger.info("Initialize weight to %s" % init_weight[:self.s_max + 1])
         types, bounds = get_types(config_space)
 
         if not self.use_bohb_strategy:
@@ -91,7 +92,7 @@ class async_mqMFES(async_mqHyperband):
             self.target_y[r] = list()
 
         # BO optimizer settings.
-        self.history_container = HistoryContainer(task_id=self.method_name)
+        self.history = History(task_id=self.method_name)
         self.sls_max_steps = None
         self.n_sls_iterations = 5
         self.sls_n_steps_plateau_walk = 10
@@ -136,8 +137,9 @@ class async_mqMFES(async_mqHyperband):
         if n_iteration == self.R:
             self.incumbent_configs.append(config)
             self.incumbent_perfs.append(perf)
-            # Update history container.
-            self.history_container.add(config, perf)
+            # Update history
+            observation = Observation(config=config, objectives=[perf])
+            self.history.update_observation(observation)
 
         # Refit the ensemble surrogate model.
         configs_train = self.target_x[n_iteration] + configs_running
@@ -180,7 +182,7 @@ class async_mqMFES(async_mqHyperband):
                         next_config = config
                         break
                 if next_config is None:
-                    self.logger.warning('Cannot get a non duplicate configuration from bo candidates. '
+                    logger.warning('Cannot get a non duplicate configuration from bo candidates. '
                                         'Sample a random one.')
                     next_config = sample_configuration(self.config_space, excluded_configs=excluded_configs)
 
@@ -194,7 +196,7 @@ class async_mqMFES(async_mqHyperband):
                                          num_data=len(self.incumbent_configs))
 
         challengers = self.acq_optimizer.maximize(
-            runhistory=self.history_container,
+            runhistory=self.history,
             num_points=5000,
         )
         return challengers.challengers
@@ -308,7 +310,7 @@ class async_mqMFES(async_mqHyperband):
                 old_weights.append(_weight)
             new_weights = old_weights.copy()
 
-        self.logger.info('[%s] %d-th Updating weights: %s' % (
+        logger.info('[%s] %d-th Updating weights: %s' % (
             self.weight_method, self.weight_changed_cnt, str(new_weights)))
 
         # Assign the weight to each basic surrogate.
@@ -322,9 +324,8 @@ class async_mqMFES(async_mqHyperband):
         if not os.path.exists(dir_path):
             os.makedirs(dir_path)
         np.save(os.path.join(dir_path, file_name), np.asarray(self.hist_weights))
-        self.logger.info('update_weight() cost %.2fs. new weights are saved to %s'
+        logger.info('update_weight() cost %.2fs. new weights are saved to %s'
                          % (time.time() - start_time, os.path.join(dir_path, file_name)))
 
     def get_weights(self):
         return self.hist_weights
-
