@@ -3,12 +3,11 @@
 import time
 import numpy as np
 from math import log, ceil
+from openbox import logger
 from openbox.apps.multi_fidelity.async_mq_base_facade import async_mqBaseFacade
 from openbox.apps.multi_fidelity.utils import RUNNING, COMPLETED, PROMOTED
 from openbox.apps.multi_fidelity.utils import sample_configuration
-
 from openbox.utils.config_space import ConfigurationSpace
-from openbox.utils.constants import MAXINT
 
 
 class async_mqSuccessiveHalving_v0(async_mqBaseFacade):
@@ -74,7 +73,7 @@ class async_mqSuccessiveHalving_v0(async_mqBaseFacade):
                 num_promoted=0,
             )
             self.bracket.append(rung)
-        self.logger.info('Init bracket: %s.' % str(self.bracket))
+        logger.info('Init bracket: %s.' % str(self.bracket))
 
     def get_job(self):
         """
@@ -98,7 +97,7 @@ class async_mqSuccessiveHalving_v0(async_mqBaseFacade):
             candidate_jobs = complete_jobs[0: int(len(complete_jobs) / self.eta)]
             for job_id, job in candidate_jobs:
                 job_status, config, perf, extra_conf = job
-                if not (job_status == COMPLETED and perf < MAXINT):
+                if not (job_status == COMPLETED and perf < np.inf):
                     continue
                 # check if config already exists in upper rungs
                 exist = False
@@ -112,10 +111,10 @@ class async_mqSuccessiveHalving_v0(async_mqBaseFacade):
                     next_n_iteration = self.bracket[rung_id + 1]['n_iteration']
                     next_extra_conf = extra_conf
                     # update bracket
-                    self.logger.info('Promote job in rung %d: %s' % (rung_id, self.bracket[rung_id]['jobs'][job_id]))
+                    logger.info('Promote job in rung %d: %s' % (rung_id, self.bracket[rung_id]['jobs'][job_id]))
                     self.bracket[rung_id]['jobs'][job_id][0] = PROMOTED
                     self.bracket[rung_id]['num_promoted'] += 1
-                    new_job = [RUNNING, next_config, MAXINT, next_extra_conf]     # running perf is set to MAXINT
+                    new_job = [RUNNING, next_config, np.inf, next_extra_conf]     # running perf is set to np.inf
                     self.bracket[rung_id + 1]['jobs'].append(new_job)
                     self.bracket[rung_id + 1]['configs'].add(next_config)
                     assert len(self.bracket[rung_id + 1]['jobs']) == len(self.bracket[rung_id + 1]['configs'])
@@ -128,8 +127,8 @@ class async_mqSuccessiveHalving_v0(async_mqBaseFacade):
             next_config, next_n_iteration, next_extra_conf = self.choose_next()
             # update bracket
             rung_id = self.get_rung_id(self.bracket, next_n_iteration)
-            self.logger.info('Sample a new config: %s. Add to rung %d.' % (next_config, rung_id))
-            new_job = [RUNNING, next_config, MAXINT, next_extra_conf]   # running perf is set to MAXINT
+            logger.info('Sample a new config: %s. Add to rung %d.' % (next_config, rung_id))
+            new_job = [RUNNING, next_config, np.inf, next_extra_conf]   # running perf is set to np.inf
             self.bracket[rung_id]['jobs'].append(new_job)
             self.bracket[rung_id]['configs'].add(next_config)
             assert len(self.bracket[rung_id]['jobs']) == len(self.bracket[rung_id]['configs'])
@@ -202,7 +201,7 @@ class async_mqSuccessiveHalving_v0(async_mqBaseFacade):
         status += '=' * 46 + '\n'
         return status
 
-    def get_incumbent(self, num_inc=1):
+    def get_incumbents(self, num_inc=1):
         assert (len(self.incumbent_perfs) == len(self.incumbent_configs))
         indices = np.argsort(self.incumbent_perfs)
         configs = [self.incumbent_configs[i] for i in indices[0:num_inc]]
