@@ -3,17 +3,15 @@
 import os
 import abc
 import numpy as np
+from datetime import datetime
 
-from openbox.utils.util_funcs import check_random_state
-from openbox.utils.logging_utils import get_logger
-from openbox.utils.history_container import HistoryContainer, MOHistoryContainer, \
-    MultiStartHistoryContainer
+from openbox import logger
+from openbox.utils.util_funcs import check_random_state, deprecate_kwarg
+from openbox.utils.history import Observation, History
 from openbox.utils.constants import MAXINT, SUCCESS
-from openbox.utils.samplers import SobolSampler, LatinHypercubeSampler
+from openbox.utils.samplers import SobolSampler, LatinHypercubeSampler, HaltonSampler
 from openbox.utils.multi_objective import get_chebyshev_scalarization, NondominatedPartitioning
-from openbox.utils.config_space.util import convert_configurations_to_array
 from openbox.core.base import build_acq_func, build_optimizer, build_surrogate
-from openbox.core.base import Observation
 
 
 class Advisor(object, metaclass=abc.ABCMeta):
@@ -21,41 +19,47 @@ class Advisor(object, metaclass=abc.ABCMeta):
     Basic Advisor Class, which adopts a policy to sample a configuration.
     """
 
-    def __init__(self, config_space,
-                 num_objs=1,
-                 num_constraints=0,
-                 initial_trials=3,
-                 initial_configurations=None,
-                 init_strategy='random_explore_first',
-                 history_bo_data=None,
-                 rand_prob=0.1,
-                 optimization_strategy='bo',
-                 surrogate_type='auto',
-                 acq_type='auto',
-                 acq_optimizer_type='auto',
-                 ref_point=None,
-                 output_dir='logs',
-                 task_id='default_task_id',
-                 random_state=None,
-                 **kwargs):
+    @deprecate_kwarg('num_objs', 'num_objectives', 'a future version')
+    def __init__(
+            self,
+            config_space,
+            num_objectives=1,
+            num_constraints=0,
+            initial_trials=3,
+            initial_configurations=None,
+            init_strategy='random_explore_first',
+            transfer_learning_history=None,
+            rand_prob=0.1,
+            optimization_strategy='bo',
+            surrogate_type='auto',
+            acq_type='auto',
+            acq_optimizer_type='auto',
+            ref_point=None,
+            output_dir='logs',
+            task_id='OpenBox',
+            random_state=None,
+            logger_kwargs: dict = None,
+            **kwargs,
+    ):
 
-        # Create output (logging) directory.
-        # Init logging module.
-        # Random seed generator.
-        self.num_objs = num_objs
+        self.timestamp = datetime.now().strftime('%Y-%m-%d-%H-%M-%S-%f')
+        self.num_objectives = num_objectives
         self.num_constraints = num_constraints
         self.init_strategy = init_strategy
         self.output_dir = output_dir
         self.task_id = task_id
         self.rng = check_random_state(random_state)
-        self.logger = get_logger(self.__class__.__name__)
+
+        _logger_kwargs = {'name': task_id, 'logdir': output_dir}
+        _logger_kwargs.update(logger_kwargs or {})
+        logger.init(**_logger_kwargs)
 
         # Basic components in Advisor.
         self.rand_prob = rand_prob
         self.optimization_strategy = optimization_strategy
 
         # Init the basic ingredients in Bayesian optimization.
-        self.history_bo_data = history_bo_data
+        self.transfer_learning_history = transfer_learning_history
         self.surrogate_type = surrogate_type
         self.constraint_surrogate_type = None
         self.acq_type = acq_type
@@ -66,12 +70,11 @@ class Advisor(object, metaclass=abc.ABCMeta):
         self.config_space.seed(self.config_space_seed)
         self.ref_point = ref_point
 
-        # init history container
-        if self.num_objs == 1:
-            self.history_container = HistoryContainer(task_id, self.num_constraints, config_space=self.config_space)
-        else:  # multi-objectives
-            self.history_container = MOHistoryContainer(task_id, self.num_objs, self.num_constraints,
-                                                        config_space=self.config_space, ref_point=ref_point)
+        # init history
+        self.history = History(
+            task_id=task_id, num_objectives=num_objectives, num_constraints=num_constraints, config_space=config_space,
+            ref_point=ref_point, meta_info=None,  # todo: add meta info
+        )
 
         # initial design
         if initial_configurations is not None and len(initial_configurations) > 0:
@@ -109,25 +112,26 @@ class Advisor(object, metaclass=abc.ABCMeta):
         info_str = ''
 
         if self.surrogate_type == 'auto':
-            self.auto_alter_model = True
+            use_tl = self.transfer_learning_history is not None
+            self.auto_alter_model = True if not use_tl else False
             if n_total_hp >= 100:
                 self.optimization_strategy = 'random'
                 self.surrogate_type = 'prf'  # for setup procedure
             elif n_total_hp >= 10:
-                self.surrogate_type = 'prf'
+                self.surrogate_type = 'prf' if not use_tl else 'tlbo_rgpe_prf'
             elif n_cat_hp > n_cont_hp:
-                self.surrogate_type = 'prf'
+                self.surrogate_type = 'prf' if not use_tl else 'tlbo_rgpe_prf'
             else:
-                self.surrogate_type = 'gp'
+                self.surrogate_type = 'gp' if not use_tl else 'tlbo_rgpe_gp'
             info_str += ' surrogate_type: %s.' % self.surrogate_type
 
         if self.acq_type == 'auto':
-            if self.num_objs == 1:  # single objective
+            if self.num_objectives == 1:  # single objective
                 if self.num_constraints == 0:
                     self.acq_type = 'ei'
                 else:   # with constraints
                     self.acq_type = 'eic'
-            elif self.num_objs <= 4:    # multi objective (<=4)
+            elif self.num_objectives <= 4:    # multi objective (<=4)
                 if self.num_constraints == 0:
                     self.acq_type = 'ehvi'
                 else:   # with constraints
@@ -149,38 +153,37 @@ class Advisor(object, metaclass=abc.ABCMeta):
             info_str += ' acq_optimizer_type: %s.' % self.acq_optimizer_type
 
         if info_str != '':
-            info_str = '=== [BO auto selection] ===' + info_str
-            self.logger.info(info_str)
+            info_str = '[BO auto selection] ' + info_str
+            logger.info(info_str)
 
-    def alter_model(self, history_container):
+    def alter_model(self, history: History):
         if not self.auto_alter_model:
             return
 
-        num_config_evaluated = len(history_container.configurations)
-        num_config_successful = len(history_container.successful_perfs)
+        num_config_evaluated = len(history)
 
         if num_config_evaluated == 300:
             if self.surrogate_type == 'gp':
                 self.surrogate_type = 'prf'
-                self.logger.info('n_observations=300, change surrogate model from GP to PRF!')
+                logger.info('n_observations=300, change surrogate model from GP to PRF!')
                 if self.acq_optimizer_type == 'random_scipy':
                     self.acq_optimizer_type = 'local_random'
-                    self.logger.info('n_observations=300, change acq optimizer from random_scipy to local_random!')
+                    logger.info('n_observations=300, change acq optimizer from random_scipy to local_random!')
                 self.setup_bo_basics()
 
     def check_setup(self):
         """
-        Check optimization_strategy, num_objs, num_constraints, acq_type, surrogate_type.
+        Check optimization_strategy, num_objectives, num_constraints, acq_type, surrogate_type.
         Returns
         -------
         None
         """
         assert self.optimization_strategy in ['bo', 'random']
-        assert isinstance(self.num_objs, int) and self.num_objs >= 1
+        assert isinstance(self.num_objectives, int) and self.num_objectives >= 1
         assert isinstance(self.num_constraints, int) and self.num_constraints >= 0
 
         # single objective
-        if self.num_objs == 1:
+        if self.num_objectives == 1:
             if self.num_constraints == 0:
                 assert self.acq_type in ['ei', 'eips', 'logei', 'pi', 'lcb', 'lpei', ]
             else:  # with constraints
@@ -194,8 +197,8 @@ class Advisor(object, metaclass=abc.ABCMeta):
                 assert self.acq_type in ['ehvi', 'mesmo', 'usemo', 'parego']
                 if self.acq_type == 'mesmo' and self.surrogate_type != 'gp_rbf':
                     self.surrogate_type = 'gp_rbf'
-                    self.logger.warning('Surrogate model has changed to Gaussian Process with RBF kernel '
-                                        'since MESMO is used. Surrogate_type should be set to \'gp_rbf\'.')
+                    logger.warning('Surrogate model has changed to Gaussian Process with RBF kernel '
+                                   'since MESMO is used. Surrogate_type should be set to \'gp_rbf\'.')
             else:  # with constraints
                 assert self.acq_type in ['ehvic', 'mesmoc', 'mesmoc2']
                 if self.constraint_surrogate_type is None:
@@ -205,17 +208,26 @@ class Advisor(object, metaclass=abc.ABCMeta):
                         self.constraint_surrogate_type = 'gp'
                 if self.acq_type == 'mesmoc' and self.surrogate_type != 'gp_rbf':
                     self.surrogate_type = 'gp_rbf'
-                    self.logger.warning('Surrogate model has changed to Gaussian Process with RBF kernel '
-                                        'since MESMOC is used. Surrogate_type should be set to \'gp_rbf\'.')
+                    logger.warning('Surrogate model has changed to Gaussian Process with RBF kernel '
+                                   'since MESMOC is used. Surrogate_type should be set to \'gp_rbf\'.')
                 if self.acq_type == 'mesmoc' and self.constraint_surrogate_type != 'gp_rbf':
                     self.surrogate_type = 'gp_rbf'
-                    self.logger.warning('Constraint surrogate model has changed to Gaussian Process with RBF kernel '
-                                        'since MESMOC is used. Surrogate_type should be set to \'gp_rbf\'.')
+                    logger.warning('Constraint surrogate model has changed to Gaussian Process with RBF kernel '
+                                   'since MESMOC is used. Surrogate_type should be set to \'gp_rbf\'.')
 
             # Check reference point is provided for EHVI methods
             if 'ehvi' in self.acq_type and self.ref_point is None:
                 raise ValueError('Must provide reference point to use EHVI method!')
 
+        # transfer learning
+        if self.transfer_learning_history is not None:
+            if not (self.num_objectives == 1 and self.num_constraints == 0):
+                raise NotImplementedError('Currently, transfer learning is only supported for single objective '
+                                          'optimization without constraints.')
+            surrogate_str = self.surrogate_type.split('_')
+            assert len(surrogate_str) == 3 and surrogate_str[0] == 'tlbo'
+            assert surrogate_str[1] in ['rgpe', 'sgpr', 'topov3']  # todo: 'mfgpe'
+
     def setup_bo_basics(self):
         """
         Prepare the basic BO components.
@@ -223,17 +235,17 @@ class Advisor(object, metaclass=abc.ABCMeta):
         -------
         An optimizer object.
         """
-        if self.num_objs == 1 or self.acq_type == 'parego':
+        if self.num_objectives == 1 or self.acq_type == 'parego':
             self.surrogate_model = build_surrogate(func_str=self.surrogate_type,
                                                    config_space=self.config_space,
                                                    rng=self.rng,
-                                                   history_hpo_data=self.history_bo_data)
+                                                   transfer_learning_history=self.transfer_learning_history)
         else:  # multi-objectives
             self.surrogate_model = [build_surrogate(func_str=self.surrogate_type,
                                                     config_space=self.config_space,
                                                     rng=self.rng,
-                                                    history_hpo_data=self.history_bo_data)
-                                    for _ in range(self.num_objs)]
+                                                    transfer_learning_history=self.transfer_learning_history)
+                                    for _ in range(self.num_objectives)]
 
         if self.num_constraints > 0:
             self.constraint_models = [build_surrogate(func_str=self.constraint_surrogate_type,
@@ -272,24 +284,38 @@ class Advisor(object, metaclass=abc.ABCMeta):
         num_random_config = self.init_num - 1
         if init_strategy == 'random':
             initial_configs = self.sample_random_configs(self.init_num)
-            return initial_configs
         elif init_strategy == 'default':
             initial_configs = [default_config] + self.sample_random_configs(num_random_config)
-            return initial_configs
         elif init_strategy == 'random_explore_first':
             candidate_configs = self.sample_random_configs(100)
-            return self.max_min_distance(default_config, candidate_configs, num_random_config)
+            initial_configs = self.max_min_distance(default_config, candidate_configs, num_random_config)
         elif init_strategy == 'sobol':
             sobol = SobolSampler(self.config_space, num_random_config, random_state=self.rng)
             initial_configs = [default_config] + sobol.generate(return_config=True)
-            return initial_configs
         elif init_strategy == 'latin_hypercube':
             lhs = LatinHypercubeSampler(self.config_space, num_random_config, criterion='maximin')
             initial_configs = [default_config] + lhs.generate(return_config=True)
-            return initial_configs
+        elif init_strategy == 'halton':
+            halton = HaltonSampler(self.config_space, num_random_config, random_state=self.rng)
+            initial_configs = [default_config] + halton.generate(return_config=True)
         else:
             raise ValueError('Unknown initial design strategy: %s.' % init_strategy)
 
+        valid_configs = []
+        for config in initial_configs:
+            try:
+                config.is_valid_configuration()
+            except ValueError:
+                continue
+            valid_configs.append(config)
+        if len(valid_configs) != len(initial_configs):
+            logger.warning('Only %d/%d valid configurations are generated for initial design strategy: %s. '
+                                'Add more random configurations.'
+                                % (len(valid_configs), len(initial_configs), init_strategy))
+            num_random_config = self.init_num - len(valid_configs)
+            valid_configs += self.sample_random_configs(num_random_config, excluded_configs=valid_configs)
+        return valid_configs
+
     def max_min_distance(self, default_config, src_configs, num):
         min_dis = list()
         initial_configs = list()
@@ -313,50 +339,53 @@ class Advisor(object, metaclass=abc.ABCMeta):
 
         return initial_configs
 
-    def get_suggestion(self, history_container=None, return_list=False):
+    def get_suggestion(self, history: History = None, return_list: bool = False):
         """
         Generate a configuration (suggestion) for this query.
         Returns
         -------
         A configuration.
         """
-        if history_container is None:
-            history_container = self.history_container
+        if history is None:
+            history = self.history
 
-        self.alter_model(history_container)
+        self.alter_model(history)
 
-        num_config_evaluated = len(history_container.configurations)
-        num_config_successful = len(history_container.successful_perfs)
+        num_config_evaluated = len(history)
+        num_config_successful = history.get_success_count()
 
         if num_config_evaluated < self.init_num:
-            return self.initial_configurations[num_config_evaluated]
-
+            res = self.initial_configurations[num_config_evaluated]
+            return [res] if return_list else res
         if self.optimization_strategy == 'random':
-            return self.sample_random_configs(1, history_container)[0]
+            res = self.sample_random_configs(1, history)[0]
+            return [res] if return_list else res
 
         if (not return_list) and self.rng.random() < self.rand_prob:
-            self.logger.info('Sample random config. rand_prob=%f.' % self.rand_prob)
-            return self.sample_random_configs(1, history_container)[0]
+            logger.info('Sample random config. rand_prob=%f.' % self.rand_prob)
+            res = self.sample_random_configs(1, history)[0]
+            return [res] if return_list else res
 
-        X = convert_configurations_to_array(history_container.configurations)
-        Y = history_container.get_transformed_perfs(transform=None)
-        cY = history_container.get_transformed_constraint_perfs(transform='bilog')
+        X = history.get_config_array(transform='scale')
+        Y = history.get_objectives(transform='infeasible')
+        cY = history.get_constraints(transform='bilog')
 
         if self.optimization_strategy == 'bo':
             if num_config_successful < max(self.init_num, 1):
-                self.logger.warning('No enough successful initial trials! Sample random configuration.')
-                return self.sample_random_configs(1, history_container)[0]
+                logger.warning('No enough successful initial trials! Sample random configuration.')
+                res = self.sample_random_configs(1, history)[0]
+                return [res] if return_list else res
 
             # train surrogate model
-            if self.num_objs == 1:
-                self.surrogate_model.train(X, Y)
+            if self.num_objectives == 1:
+                self.surrogate_model.train(X, Y[:, 0])
             elif self.acq_type == 'parego':
-                weights = self.rng.random_sample(self.num_objs)
+                weights = self.rng.random_sample(self.num_objectives)
                 weights = weights / np.sum(weights)
                 scalarized_obj = get_chebyshev_scalarization(weights, Y)
                 self.surrogate_model.train(X, scalarized_obj(Y))
             else:  # multi-objectives
-                for i in range(self.num_objs):
+                for i in range(self.num_objectives):
                     self.surrogate_model[i].train(X, Y[:, i])
 
             # train constraint model
@@ -364,21 +393,21 @@ class Advisor(object, metaclass=abc.ABCMeta):
                 self.constraint_models[i].train(X, cY[:, i])
 
             # update acquisition function
-            if self.num_objs == 1:
-                incumbent_value = history_container.get_incumbents()[0][1]
+            if self.num_objectives == 1:
+                incumbent_value = history.get_incumbent_value()
                 self.acquisition_function.update(model=self.surrogate_model,
                                                  constraint_models=self.constraint_models,
                                                  eta=incumbent_value,
                                                  num_data=num_config_evaluated)
             else:  # multi-objectives
-                mo_incumbent_value = history_container.get_mo_incumbent_value()
+                mo_incumbent_values = history.get_mo_incumbent_values()
                 if self.acq_type == 'parego':
                     self.acquisition_function.update(model=self.surrogate_model,
                                                      constraint_models=self.constraint_models,
-                                                     eta=scalarized_obj(np.atleast_2d(mo_incumbent_value)),
+                                                     eta=scalarized_obj(np.atleast_2d(mo_incumbent_values)),
                                                      num_data=num_config_evaluated)
                 elif self.acq_type.startswith('ehvi'):
-                    partitioning = NondominatedPartitioning(self.num_objs, Y)
+                    partitioning = NondominatedPartitioning(self.num_objectives, Y)
                     cell_bounds = partitioning.get_hypercell_bounds(ref_point=self.ref_point)
                     self.acquisition_function.update(model=self.surrogate_model,
                                                      constraint_models=self.constraint_models,
@@ -388,23 +417,23 @@ class Advisor(object, metaclass=abc.ABCMeta):
                     self.acquisition_function.update(model=self.surrogate_model,
                                                      constraint_models=self.constraint_models,
                                                      constraint_perfs=cY,  # for MESMOC
-                                                     eta=mo_incumbent_value,
+                                                     eta=mo_incumbent_values,
                                                      num_data=num_config_evaluated,
                                                      X=X, Y=Y)
 
             # optimize acquisition function
-            challengers = self.optimizer.maximize(runhistory=history_container,
+            challengers = self.optimizer.maximize(runhistory=history,
                                                   num_points=5000)
             if return_list:
                 # Caution: return_list doesn't contain random configs sampled according to rand_prob
                 return challengers.challengers
 
             for config in challengers.challengers:
-                if config not in history_container.configurations:
+                if config not in history.configurations:
                     return config
-            self.logger.warning('Cannot get non duplicate configuration from BO candidates (len=%d). '
+            logger.warning('Cannot get non duplicate configuration from BO candidates (len=%d). '
                                 'Sample random config.' % (len(challengers.challengers), ))
-            return self.sample_random_configs(1, history_container)[0]
+            return self.sample_random_configs(1, history)[0]
         else:
             raise ValueError('Unknown optimization strategy: %s.' % self.optimization_strategy)
 
@@ -419,23 +448,23 @@ class Advisor(object, metaclass=abc.ABCMeta):
         -------
 
         """
-        return self.history_container.update_observation(observation)
+        return self.history.update_observation(observation)
 
-    def sample_random_configs(self, num_configs=1, history_container=None, excluded_configs=None):
+    def sample_random_configs(self, num_configs=1, history=None, excluded_configs=None):
         """
         Sample a batch of random configurations.
         Parameters
         ----------
         num_configs
 
-        history_container
+        history
 
         Returns
         -------
 
         """
-        if history_container is None:
-            history_container = self.history_container
+        if history is None:
+            history = self.history
         if excluded_configs is None:
             excluded_configs = set()
 
@@ -445,40 +474,32 @@ class Advisor(object, metaclass=abc.ABCMeta):
         while len(configs) < num_configs:
             config = self.config_space.sample_configuration()
             sample_cnt += 1
-            if config not in (history_container.configurations + configs) and config not in excluded_configs:
+            if config not in (history.configurations + configs) and config not in excluded_configs:
                 configs.append(config)
                 sample_cnt = 0
                 continue
             if sample_cnt >= max_sample_cnt:
-                self.logger.warning('Cannot sample non duplicate configuration after %d iterations.' % max_sample_cnt)
+                logger.warning('Cannot sample non duplicate configuration after %d iterations.' % max_sample_cnt)
                 configs.append(config)
                 sample_cnt = 0
         return configs
 
     def get_history(self):
-        return self.history_container
+        return self.history
 
-    def save_history(self, dir_path: str = None, file_name: str = None):
+    def save_json(self, filename: str = None):
         """
         Save history to a json file.
         """
-        if dir_path is None:
-            dir_path = os.path.join(self.output_dir, 'bo_history')
-        if not os.path.exists(dir_path):
-            os.makedirs(dir_path)
-        if file_name is None:
-            file_name = 'bo_history_%s.json' % self.task_id
-        self.history_container.save_json(os.path.join(dir_path, file_name))
+        if filename is None:
+            filename = os.path.join(self.output_dir, f'history/{self.task_id}/history_{self.timestamp}.json')
+        self.history.save_json(filename)
 
-    def load_history_from_json(self, file_name=None):
+    def load_json(self, filename: str):
         """
         Load history from a json file.
         """
-        if file_name is None:
-            file_name = os.path.join(self.output_dir, 'bo_history', 'bo_history_%s.json' % self.task_id)
-        if not os.path.exists(file_name):
-            raise FileNotFoundError('History file not found: %s' % file_name)
-        self.history_container.load_history_from_json(file_name)
+        self.history = History.load_json(filename, self.config_space)
 
     def get_suggestions(self):
         raise NotImplementedError
