@@ -1,7 +1,6 @@
 # License: MIT
 
 import traceback
-import logging
 import ConfigSpace
 import ConfigSpace.hyperparameters
 import ConfigSpace.util
@@ -9,24 +8,33 @@ import numpy as np
 import scipy.stats as sps
 import statsmodels.api as sm
 
+from openbox import logger
 from openbox.utils.util_funcs import check_random_state
-from openbox.utils.history_container import HistoryContainer
-from openbox.core.base import Observation
-from openbox.utils.config_space.util import convert_configurations_to_array
+from openbox.utils.history import Observation, History
 
 
 class TPE_Advisor:
     # TODO：Add warm start
-    def __init__(self, config_space,
-                 min_points_in_model=None,
-                 top_n_percent=15,
-                 num_samples=64,
-                 random_fraction=1 / 3,
-                 bandwidth_factor=3,
-                 min_bandwidth=1e-3,
-                 task_id='default_task_id',
-                 output_dir='logs',
-                 random_state=None):
+    def __init__(
+            self,
+            config_space,
+            min_points_in_model=None,
+            top_n_percent=15,
+            num_samples=64,
+            random_fraction=1 / 3,
+            bandwidth_factor=3,
+            min_bandwidth=1e-3,
+            task_id='OpenBox',
+            output_dir='logs',
+            random_state=None,
+            logger_kwargs: dict = None,
+    ):
+        self.task_id = task_id
+        self.output_dir = output_dir
+        _logger_kwargs = {'name': task_id, 'logdir': output_dir}
+        _logger_kwargs.update(logger_kwargs or {})
+        logger.init(**_logger_kwargs)
+
         self.rng = check_random_state(random_state)
         self.config_space = config_space
         self.config_space.seed(self.rng.randint(100000))
@@ -35,8 +43,10 @@ class TPE_Advisor:
         self.bw_factor = bandwidth_factor
         self.min_bandwidth = min_bandwidth
 
-        self.history_container = HistoryContainer(task_id, config_space=config_space)
-        self.output_dir = output_dir
+        self.history = History(
+            task_id=task_id, num_objectives=1, num_constraints=0, config_space=config_space,
+            ref_point=None, meta_info=None,  # todo: add meta info
+        )
 
         self.min_points_in_model = min_points_in_model
         if min_points_in_model is None:
@@ -68,26 +78,25 @@ class TPE_Advisor:
 
         self.good_config_rankings = dict()
         self.kde_models = dict()
-        self.logger = logging.getLogger(self.__class__.__name__)
 
     def update_observation(self, observation: Observation):
-        self.history_container.update_observation(observation)
+        self.history.update_observation(observation)
 
-    def get_suggestion(self, history_container=None):
-        if history_container is None:
-            history_container = self.history_container
+    def get_suggestion(self, history=None):
+        if history is None:
+            history = self.history
 
         # use default as first config
-        num_config_evaluated = len(history_container.configurations)
+        num_config_evaluated = len(history)
         if num_config_evaluated == 0:
             return self.config_space.get_default_configuration()
 
         # fit
-        self.fit_kde_models(history_container)
+        self.fit_kde_models(history)
 
         # If no model is available, sample random config
         if len(self.kde_models.keys()) == 0 or self.rng.rand() < self.random_fraction:
-            return self.sample_random_configs(1, history_container)[0]
+            return self.sample_random_configs(1, history)[0]
 
         best = np.inf
         best_vector = None
@@ -114,10 +123,10 @@ class TPE_Advisor:
                         try:
                             vector.append(sps.truncnorm.rvs(-m / bw, (1 - m) / bw, loc=m, scale=bw))
                         except:
-                            self.logger.warning(
+                            logger.warning(
                                 "Truncated Normal failed for:\ndatum=%s\nbandwidth=%s\nfor entry with value %s" % (
                                     datum, kde_good.bw, m))
-                            self.logger.warning("data in the KDE:\n%s" % kde_good.data)
+                            logger.warning("data in the KDE:\n%s" % kde_good.data)
                     else:
 
                         if self.rng.rand() < (1 - bw):
@@ -127,11 +136,11 @@ class TPE_Advisor:
                 val = minimize_me(vector)
 
                 if not np.isfinite(val):
-                    self.logger.warning('sampled vector: %s has EI value %s' % (vector, val))
-                    self.logger.warning("data in the KDEs:\n%s\n%s" % (kde_good.data, kde_bad.data))
-                    self.logger.warning("bandwidth of the KDEs:\n%s\n%s" % (kde_good.bw, kde_bad.bw))
-                    self.logger.warning("l(x) = %s" % (l(vector)))
-                    self.logger.warning("g(x) = %s" % (g(vector)))
+                    logger.warning('sampled vector: %s has EI value %s' % (vector, val))
+                    logger.warning("data in the KDEs:\n%s\n%s" % (kde_good.data, kde_bad.data))
+                    logger.warning("bandwidth of the KDEs:\n%s\n%s" % (kde_good.bw, kde_bad.bw))
+                    logger.warning("l(x) = %s" % (l(vector)))
+                    logger.warning("g(x) = %s" % (g(vector)))
 
                     # right now, this happens because a KDE does not contain all values for a categorical parameter
                     # this cannot be fixed with the statsmodels KDE, so for now, we are just going to evaluate this one
@@ -145,11 +154,11 @@ class TPE_Advisor:
                     best_vector = vector
 
             if best_vector is None:
-                self.logger.debug(
+                logger.debug(
                     "Sampling based optimization with %i samples failed -> using random configuration" % self.num_samples)
-                config = self.sample_random_configs(1, history_container)[0]
+                config = self.sample_random_configs(1, history)[0]
             else:
-                self.logger.debug(
+                logger.debug(
                     'best_vector: {}, {}, {}, {}'.format(best_vector, best, l(best_vector), g(best_vector)))
                 for i, hp_value in enumerate(best_vector):
                     if isinstance(
@@ -162,17 +171,17 @@ class TPE_Advisor:
                 try:
                     config = ConfigSpace.Configuration(self.config_space, vector=best_vector)
                 except Exception as e:
-                    self.logger.warning(("=" * 50 + "\n") * 3 +
+                    logger.warning(("=" * 50 + "\n") * 3 +
                                         "Error converting configuration:\n%s" % best_vector +
                                         "\n here is a traceback:" +
                                         traceback.format_exc())
                     raise e
 
         except:
-            self.logger.warning(
+            logger.warning(
                 "Sampling based optimization with %i samples failed\n %s \nUsing random configuration" % (
                     self.num_samples, traceback.format_exc()))
-            config = self.sample_random_configs(1, history_container)[0]
+            config = self.sample_random_configs(1, history)[0]
 
         return config
 
@@ -205,15 +214,15 @@ class TPE_Advisor:
             return_array[i, :] = datum
         return return_array
 
-    def fit_kde_models(self, history_container):
-        num_config_successful = len(history_container.successful_perfs)
+    def fit_kde_models(self, history):
+        num_config_successful = history.get_success_count()
         if num_config_successful <= self.min_points_in_model - 1:
-            self.logger.debug("Only %i run(s) available, need more than %s -> can't build model!" % (
+            logger.debug("Only %i run(s) available, need more than %s -> can't build model!" % (
                 num_config_successful, self.min_points_in_model + 1))
             return
 
-        train_configs = convert_configurations_to_array(history_container.configurations)
-        train_losses = history_container.get_transformed_perfs(transform=None)
+        train_configs = history.get_config_array(transform='scale')
+        train_losses = history.get_objectives(transform='infeasible').reshape(-1)
 
         n_good = max(self.min_points_in_model, (self.top_n_percent * train_configs.shape[0]) // 100)
         # n_bad = min(max(self.min_points_in_model, ((100-self.top_n_percent)*train_configs.shape[0])//100), 10)
@@ -250,25 +259,25 @@ class TPE_Advisor:
         }
 
         # update probs for the categorical parameters for later sampling
-        self.logger.debug(
+        logger.debug(
             'done building a new model based on %i/%i split\nBest loss for this budget:%f\n\n\n\n\n' % (
                 n_good, n_bad, np.min(train_losses)))
 
-    def sample_random_configs(self, num_configs=1, history_container=None):
+    def sample_random_configs(self, num_configs=1, history=None):
         """
         Sample a batch of random configurations.
         Parameters
         ----------
         num_configs
 
-        history_container
+        history
 
         Returns
         -------
 
         """
-        if history_container is None:
-            history_container = self.history_container
+        if history is None:
+            history = self.history
 
         configs = list()
         sample_cnt = 0
@@ -276,15 +285,15 @@ class TPE_Advisor:
         while len(configs) < num_configs:
             config = self.config_space.sample_configuration()
             sample_cnt += 1
-            if config not in (history_container.configurations + configs):
+            if config not in (history.configurations + configs):
                 configs.append(config)
                 sample_cnt = 0
                 continue
             if sample_cnt >= max_sample_cnt:
-                self.logger.warning('Cannot sample non duplicate configuration after %d iterations.' % max_sample_cnt)
+                logger.warning('Cannot sample non duplicate configuration after %d iterations.' % max_sample_cnt)
                 configs.append(config)
                 sample_cnt = 0
         return configs
 
     def get_history(self):
-        return self.history_container
+        return self.history
