@@ -4,17 +4,18 @@ import sys
 import time
 import traceback
 from typing import List
-from collections import OrderedDict
-
-from openbox.utils.constants import MAXINT, SUCCESS, FAILED, TIMEOUT
-from openbox.core.computation.parallel_process import ParallelEvaluation
 from multiprocessing import Lock
+import numpy as np
+
+from openbox import logger
+from openbox.utils.constants import SUCCESS, FAILED, TIMEOUT
+from openbox.core.computation.parallel_process import ParallelEvaluation
 from openbox.utils.limit import time_limit, TimeoutException
-from openbox.utils.util_funcs import get_result
+from openbox.utils.util_funcs import parse_result, deprecate_kwarg
 from openbox.core.sync_batch_advisor import SyncBatchAdvisor
 from openbox.core.async_batch_advisor import AsyncBatchAdvisor
 from openbox.core.ea_advisor import EA_Advisor
-from openbox.core.base import Observation
+from openbox.utils.history import Observation, History
 from openbox.optimizer.base import BOBase
 
 
@@ -29,72 +30,79 @@ def wrapper(param):
         if timeout_status:
             raise TimeoutException('Timeout: time limit for this evaluation is %.1fs' % time_limit_per_trial)
         else:
-            objs, constraints = get_result(_result)
+            objectives, constraints, extra_info = parse_result(_result)
     except Exception as e:
         if isinstance(e, TimeoutException):
             trial_state = TIMEOUT
         else:
             traceback.print_exc(file=sys.stdout)
             trial_state = FAILED
-        objs = None
+        objectives = None
         constraints = None
+        extra_info = None
     elapsed_time = time.time() - start_time
     return Observation(
-        config=config, objs=objs, constraints=constraints,
-        trial_state=trial_state, elapsed_time=elapsed_time,
+        config=config, objectives=objectives, constraints=constraints,
+        trial_state=trial_state, elapsed_time=elapsed_time, extra_info=extra_info,
     )
 
 
 class pSMBO(BOBase):
-    def __init__(self, objective_function, config_space,
-                 parallel_strategy='async',
-                 batch_size=4,
-                 batch_strategy='default',
-                 num_constraints=0,
-                 num_objs=1,
-                 sample_strategy: str = 'bo',
-                 max_runs=200,
-                 time_limit_per_trial=180,
-                 surrogate_type='auto',
-                 acq_type='auto',
-                 acq_optimizer_type='auto',
-                 initial_runs=3,
-                 init_strategy='random_explore_first',
-                 initial_configurations=None,
-                 ref_point=None,
-                 history_bo_data: List[OrderedDict] = None,
-                 logging_dir='logs',
-                 task_id='default_task_id',
-                 random_state=None,
-                 advisor_kwargs: dict = None,
-                 ):
+    @deprecate_kwarg('num_objs', 'num_objectives', 'a future version')
+    def __init__(
+            self,
+            objective_function,
+            config_space,
+            num_objectives=1,
+            num_constraints=0,
+            parallel_strategy='async',
+            batch_size=4,
+            batch_strategy='default',
+            sample_strategy: str = 'bo',
+            max_runs=200,
+            time_limit_per_trial=180,
+            surrogate_type='auto',
+            acq_type='auto',
+            acq_optimizer_type='auto',
+            initial_runs=3,
+            init_strategy='random_explore_first',
+            initial_configurations=None,
+            ref_point=None,
+            transfer_learning_history: List[History] = None,
+            logging_dir='logs',
+            task_id='OpenBox',
+            random_state=None,
+            advisor_kwargs: dict = None,
+            logger_kwargs: dict = None,
+    ):
 
         if task_id is None:
             raise ValueError('Task id is not SPECIFIED. Please input task id first.')
 
-        self.num_objs = num_objs
+        self.num_objectives = num_objectives
         self.num_constraints = num_constraints
-        self.FAILED_PERF = [MAXINT] * num_objs
+        self.FAILED_PERF = [np.inf] * num_objectives
         super().__init__(objective_function, config_space, task_id=task_id, output_dir=logging_dir,
                          random_state=random_state, initial_runs=initial_runs, max_runs=max_runs,
                          sample_strategy=sample_strategy, time_limit_per_trial=time_limit_per_trial,
-                         history_bo_data=history_bo_data)
+                         transfer_learning_history=transfer_learning_history, logger_kwargs=logger_kwargs)
 
         self.parallel_strategy = parallel_strategy
         self.batch_size = batch_size
 
         advisor_kwargs = advisor_kwargs or {}
+        _logger_kwargs = {'force_init': False}  # do not init logger in advisor
         if parallel_strategy == 'sync':
             if sample_strategy in ['random', 'bo']:
                 self.config_advisor = SyncBatchAdvisor(config_space,
-                                                       num_objs=num_objs,
+                                                       num_objectives=num_objectives,
                                                        num_constraints=num_constraints,
                                                        batch_size=batch_size,
                                                        batch_strategy=batch_strategy,
                                                        initial_trials=initial_runs,
                                                        initial_configurations=initial_configurations,
                                                        init_strategy=init_strategy,
-                                                       history_bo_data=history_bo_data,
+                                                       transfer_learning_history=transfer_learning_history,
                                                        optimization_strategy=sample_strategy,
                                                        surrogate_type=surrogate_type,
                                                        acq_type=acq_type,
@@ -103,17 +111,19 @@ class pSMBO(BOBase):
                                                        task_id=task_id,
                                                        output_dir=logging_dir,
                                                        random_state=random_state,
+                                                       logger_kwargs=_logger_kwargs,
                                                        **advisor_kwargs)
             elif sample_strategy == 'ea':
-                assert num_objs == 1 and num_constraints == 0
+                assert num_objectives == 1 and num_constraints == 0
                 self.config_advisor = EA_Advisor(config_space,
-                                                 num_objs=num_objs,
+                                                 num_objectives=num_objectives,
                                                  num_constraints=num_constraints,
                                                  optimization_strategy=sample_strategy,
                                                  batch_size=batch_size,
                                                  task_id=task_id,
                                                  output_dir=logging_dir,
                                                  random_state=random_state,
+                                                 logger_kwargs=_logger_kwargs,
                                                  **advisor_kwargs)
             else:
                 raise ValueError('Unknown sample_strategy: %s' % sample_strategy)
@@ -121,14 +131,14 @@ class pSMBO(BOBase):
             self.advisor_lock = Lock()
             if sample_strategy in ['random', 'bo']:
                 self.config_advisor = AsyncBatchAdvisor(config_space,
-                                                        num_objs=num_objs,
+                                                        num_objectives=num_objectives,
                                                         num_constraints=num_constraints,
                                                         batch_size=batch_size,
                                                         batch_strategy=batch_strategy,
                                                         initial_trials=initial_runs,
                                                         initial_configurations=initial_configurations,
                                                         init_strategy=init_strategy,
-                                                        history_bo_data=history_bo_data,
+                                                        transfer_learning_history=transfer_learning_history,
                                                         optimization_strategy=sample_strategy,
                                                         surrogate_type=surrogate_type,
                                                         acq_type=acq_type,
@@ -137,17 +147,19 @@ class pSMBO(BOBase):
                                                         task_id=task_id,
                                                         output_dir=logging_dir,
                                                         random_state=random_state,
+                                                        logger_kwargs=_logger_kwargs,
                                                         **advisor_kwargs)
             elif sample_strategy == 'ea':
-                assert num_objs == 1 and num_constraints == 0
+                assert num_objectives == 1 and num_constraints == 0
                 self.config_advisor = EA_Advisor(config_space,
-                                                 num_objs=num_objs,
+                                                 num_objectives=num_objectives,
                                                  num_constraints=num_constraints,
                                                  optimization_strategy=sample_strategy,
                                                  batch_size=batch_size,
                                                  task_id=task_id,
                                                  output_dir=logging_dir,
                                                  random_state=random_state,
+                                                 logger_kwargs=_logger_kwargs,
                                                  **advisor_kwargs)
             else:
                 raise ValueError('Unknown sample_strategy: %s' % sample_strategy)
@@ -155,16 +167,13 @@ class pSMBO(BOBase):
             raise ValueError('Invalid parallel strategy - %s.' % parallel_strategy)
 
     def callback(self, observation: Observation):
-        if observation.objs is None:
-            observation = Observation(
-                config=observation.config, objs=self.FAILED_PERF, constraints=observation.constraints,
-                trial_state=observation.trial_state, elapsed_time=observation.elapsed_time,
-            )
+        if observation.objectives is None:
+            observation.objectives = self.FAILED_PERF.copy()
         # Report the result, and remove the config from the running queue.
         with self.advisor_lock:
             # Parent process: collect the result and increment id.
             self.config_advisor.update_observation(observation)
-            self.logger.info('Update observation %d: %s.' % (self.iteration_id + 1, str(observation)))
+            logger.info('Update observation %d: %s.' % (self.iteration_id + 1, str(observation)))
             self.iteration_id += 1  # must increment id after updating
 
     # TODO: Wrong logic. Need to wait before return?
@@ -181,7 +190,7 @@ class pSMBO(BOBase):
                     time.sleep(0.3)
 
     # Asynchronously evaluate n configs
-    def async_iterate(self, n=1):
+    def async_iterate(self, n=1) -> List[Observation]:
         iter_id = 0
         res_list = list()
         with ParallelEvaluation(wrapper, n_worker=self.batch_size) as proc:
@@ -198,12 +207,8 @@ class pSMBO(BOBase):
             for res in res_list:
                 res.wait()
 
-        history = self.get_history()
-        iter_config = history.configurations[-n:]
-        iter_trial_state = history.trial_states[-n:]
-        iter_constraints = history.constraint_perfs[-n:] if self.num_constraints > 0 else None
-        iter_objs = history.perfs[-n:]     # caution: one dim if num_objs==1, different from SMBO.iterate()
-        return iter_config, iter_trial_state, iter_constraints, iter_objs
+        iter_observations = self.get_history().observations[-n:]
+        return iter_observations  # type: List[Observation]
 
     def sync_run(self):
         with ParallelEvaluation(wrapper, n_worker=self.batch_size) as proc:
@@ -213,23 +218,20 @@ class pSMBO(BOBase):
             batch_id = 0
             while batch_id < batch_num:
                 configs = self.config_advisor.get_suggestions()
-                self.logger.info('Running on %d configs in the %d-th batch.' % (len(configs), batch_id))
+                logger.info('Running on %d configs in the %d-th batch.' % (len(configs), batch_id))
                 params = [(self.objective_function, config, self.time_limit_per_trial) for config in configs]
                 # Wait all workers to complete their corresponding jobs.
                 observations = proc.parallel_execute(params)
                 # Report their results.
                 for idx, observation in enumerate(observations):
-                    if observation.objs is None:
-                        observation = Observation(
-                            config=observation.config, objs=self.FAILED_PERF, constraints=observation.constraints,
-                            trial_state=observation.trial_state, elapsed_time=observation.elapsed_time,
-                        )
+                    if observation.objectives is None:
+                        observation.objectives = self.FAILED_PERF.copy()
                     self.config_advisor.update_observation(observation)
-                    self.logger.info('In the %d-th batch [%d/%d], observation: %s.'
+                    logger.info('In the %d-th batch [%d/%d], observation: %s.'
                                      % (batch_id, idx+1, len(configs), observation))
                 batch_id += 1
 
-    def run(self):
+    def run(self) -> History:
         if self.parallel_strategy == 'async':
             self.async_run()
         else:
