@@ -1,15 +1,13 @@
 # License: MIT
 
-import logging
 import typing
 import numpy as np
 from typing import List, Optional, Tuple, Union
 
 from ConfigSpace import ConfigurationSpace
+from openbox import logger
 from openbox.surrogate.base.base_model import AbstractModel
 
-logger = logging.getLogger(__name__)
-
 
 class LightGBM(AbstractModel):
 
