@@ -1,6 +1,5 @@
 # License: MIT
 
-import logging
 import typing
 import numpy as np
 from typing import List, Optional, Tuple, Union
@@ -8,8 +7,6 @@ from typing import List, Optional, Tuple, Union
 from ConfigSpace import ConfigurationSpace
 from openbox.surrogate.base.base_model import AbstractModel
 
-logger = logging.getLogger(__name__)
-
 
 class RandomForestSurrogate(AbstractModel):
 
