@@ -2,11 +2,12 @@
 
 import numpy as np
 from typing import List
-from collections import OrderedDict
 from sklearn.model_selection import KFold
+from openbox import logger
 from openbox.surrogate.tlbo.base import BaseTLSurrogate
+from openbox.utils.history import History
 
-_scale_method = 'standardize'
+_scale_method = 'scale'
 
 
 class MFGPE(BaseTLSurrogate):
@@ -26,7 +27,7 @@ class MFGPE(BaseTLSurrogate):
         self.hist_ws = list()
         self.iteration_id = 0
 
-    def update_mf_trials(self, mf_hpo_data: List[OrderedDict]):
+    def update_mf_trials(self, mf_hpo_data: List[History]):
         if self.K == 0:
             self.K = len(mf_hpo_data) - 1  # K is the number of low-fidelity groups
             self.w = [1. / self.K] * self.K + [0.]
@@ -45,7 +46,7 @@ class MFGPE(BaseTLSurrogate):
         for train_idx, val_idx in kf.split(X):
             idxs.extend(list(val_idx))
             X_train, X_val, y_train, y_val = X[train_idx, :], X[val_idx, :], y[train_idx], y[val_idx]
-            model = self.build_single_surrogate(X_train, y_train, normalize=_scale_method)
+            model = self.build_single_surrogate(X_train, y_train)
             mu, var = model.predict(X_val)
             mu, var = mu.flatten(), var.flatten()
             _mu.extend(list(mu))
@@ -79,9 +80,9 @@ class MFGPE(BaseTLSurrogate):
         if snapshot_weight:
             self.snapshot_w = self.w
             weight_str = ','.join([('%.2f' % item) for item in self.snapshot_w])
-            self.logger.info('In iter-%d' % self.iteration_id)
+            # logger.info('In iter-%d' % self.iteration_id)
             self.target_weight.append(self.snapshot_w[-1])
-            self.logger.info('Weight: ' + str(weight_str))
+            logger.info(f'weight: {weight_str}')
             self.iteration_id += 1
             self.hist_ws.append(self.snapshot_w)
 
