@@ -1,8 +1,8 @@
 # License: MIT
 
 import numpy as np
-import traceback
-from openbox import Optimizer, sp
+from openbox import logger
+from openbox import Optimizer, space as sp
 from openbox.utils.constants import SUCCESS
 
 
@@ -11,11 +11,11 @@ def branin(config):
     x1, x2 = config['x1'], config['x2']
     y = (x2 - 5.1 / (4 * np.pi ** 2) * x1 ** 2 + 5 / np.pi * x1 - 6) ** 2 \
         + 10 * (1 - 1 / (8 * np.pi)) * np.cos(x1) + 10
-    return {'objs': (y,)}
+    return {'objectives': [y]}
 
 
 def run_test():
-    print('===== Test Start =====')
+    logger.info('===== Test Start =====')
     # Define Search Space
     space = sp.Space()
     x1 = sp.Real("x1", -5, 10, default_value=0)
@@ -34,14 +34,13 @@ def run_test():
         )
         history = opt.run()
     except Exception:
-        print(traceback.format_exc())
-        print('===== Exception in run_test()! Please check. =====')
+        logger.exception('===== Exception in run_test()! Please check. =====')
     else:
         cnt = history.trial_states.count(SUCCESS)
         if cnt == max_runs:
-            print('===== Congratulations! All trials succeeded. =====')
+            logger.info('===== Congratulations! All trials succeeded. =====')
         else:
-            print('===== %d/%d trials failed! Please check. =====' % (max_runs-cnt, max_runs))
+            logger.info('===== %d/%d trials failed! Please check. =====' % (max_runs-cnt, max_runs))
 
 
 if __name__ == '__main__':
