@@ -1,13 +1,15 @@
 # License: MIT
 
 import typing
-import logging
 import numbers
+import functools
 import numpy as np
+import numpy.random.mtrand
 
 from ConfigSpace.hyperparameters import CategoricalHyperparameter, \
     UniformFloatHyperparameter, UniformIntegerHyperparameter, Constant, \
     OrdinalHyperparameter
+from openbox import logger
 from openbox.utils.constants import MAXINT
 
 
@@ -55,37 +57,76 @@ def get_types(config_space, instance_features=None):
     return types, bounds
 
 
-def get_result(result):
+def transform_to_1d_list(x, hint='result', dtype=np.float64):
     """
-    Get objs and constraints from result returned by objective function.
-    Raise ValueError if objs is None as time_limit() function doesn't raise Exception
+    Transform a scalar, 1-d list, tuple, np.ndarray, touch.Tensor to 1-d list
+    If x is None or x is not 1-d (after squeeze), raise an error
+    """
+    assert x is not None, f'{hint} is None!'
+    x = np.asarray(x, dtype=dtype)
+    original_shape = x.shape
+    x = np.squeeze(x)  # np.squeeze requires numpy>=1.7.0
+    if x.ndim == 0:
+        x = x.reshape(1)
+    assert x.ndim == 1, f'The {hint} should be a 1-D array, but got shape: {x.shape}'
+    if x.shape != original_shape:
+        logger.warning(f'The shape of {hint} is changed from {original_shape} to {x.shape}.')
+    return x.tolist()
 
-    :param result:
-        return value from objective function
-    :return:
-        objs:
-            list/tuple of objective values
-        constraints:
-            list/tuple of constraint values or None
+
+def parse_result(result):
     """
-    number_typing_list = (int, float, np.int32, np.int64, np.float32, np.float64)
+    Parse (objectives, constraints, extra_info) from result returned by objective function.
+
+    Parameters
+    ----------
+    result: dict (or float, list, np.ndarray)
+        The result returned by objective function.
+        Dict is recommended, but we try to support other types, such as float, list, np.ndarray, etc.
+        If result is a dict, it should contain at least one key "objectives" (or "objs" for backward compatibility).
+        Optional keys: "constraints", "extra_info".
+
+    Returns
+    -------
+    objectives: list
+        The list of objectives.
+    constraints: list, optional
+        The list of constraints.
+    extra_info: dict, optional
+        The extra information.
+    """
+    constraints, extra_info = None, None
     if result is None:
         raise ValueError('result is None!')
     elif isinstance(result, dict):  # recommended usage
-        objs = result['objs']
-        if isinstance(objs, number_typing_list):
-            objs = [objs, ]
-        constraints = result.get('constraints', None)
-    elif isinstance(result, number_typing_list):
-        objs = [result, ]
-        constraints = None
-    else:
-        objs = result
-        constraints = None
+        # for backward compatibility
+        objectives = result.pop('objectives', None)
+        objs = result.pop('objs', None)  # todo: deprecated
+        if objectives is not None and objs is not None:
+            raise ValueError('"objectives" and "objs" are both provided! Please only provide "objectives".')
+        elif objectives is None and objs is None:
+            raise ValueError('"objectives" is None!')
+        elif objectives is None:
+            objectives = objs
+            logger.warning('Provide "objs" in result is deprecated and will be removed in future versions! '
+                           'Please use "objectives" instead.')
 
-    if objs is None:
-        raise ValueError('objs is None!')
-    return objs, constraints
+        # objectives is not None now
+        objectives = transform_to_1d_list(objectives, hint='objectives')
+
+        # optional keys
+        constraints = result.pop('constraints', None)
+        if constraints is not None:
+            constraints = transform_to_1d_list(constraints, hint='constraints')
+        extra_info = result.pop('extra_info', None)
+        if len(result) > 0:
+            logger.warning(f'Unused information in result: {result}')
+    else:
+        # provide only objectives
+        logger.warning(f'Provide result as <dict> that contains "objectives" is recommended, got {type(result)}')
+        objectives = transform_to_1d_list(result, hint='objectives')
+
+    return objectives, constraints, extra_info
 
 
 def check_random_state(seed):
@@ -113,7 +154,6 @@ def check_random_state(seed):
 def get_rng(
         rng: typing.Optional[typing.Union[int, np.random.RandomState]] = None,
         run_id: typing.Optional[int] = None,
-        logger: typing.Optional[logging.Logger] = None,
 ) -> typing.Tuple[int, np.random.RandomState]:
     """
     Initialize random number generator and set run_id
@@ -128,7 +168,6 @@ def get_rng(
     ----------
     rng : np.random.RandomState|int|None
     run_id : int, optional
-    logger: logging.Logger, optional
 
     Returns
     -------
@@ -136,8 +175,6 @@ def get_rng(
     np.random.RandomState
 
     """
-    if logger is None:
-        logger = logging.getLogger('GetRNG')
     # initialize random number generator
     if rng is not None and not isinstance(rng, (int, np.random.RandomState)):
         raise TypeError('Argument rng accepts only arguments of type None, int or np.random.RandomState, '
@@ -169,3 +206,26 @@ def get_rng(
         raise ValueError('This should not happen! Please contact the developers! Arguments: rng=%s of type %s and '
                          'run_id=% of type %s' % (rng, type(rng), run_id, type(run_id)))
     return run_id, rng
+
+
+def deprecate_kwarg(old_name, new_name, removed_version='a future version'):
+    """
+    Returns a decorator to deprecate a keyword argument in a function.
+    """
+    assert old_name != new_name
+
+    def decorator(func):
+        @functools.wraps(func)
+        def wrapped_func(*args, **kwargs):
+            if old_name in kwargs and new_name in kwargs:
+                raise TypeError(
+                    'Keyword argument "%s" is deprecated and will be removed in %s. '
+                    'Cannot use both kwargs "%s" and "%s"!' % (old_name, removed_version, old_name, new_name))
+
+            if old_name in kwargs:
+                logger.warning('Keyword argument "%s" is deprecated and will be removed in %s. '
+                               'Please use "%s" instead.' % (old_name, removed_version, new_name))
+                kwargs[new_name] = kwargs.pop(old_name)
+            return func(*args, **kwargs)
+        return wrapped_func
+    return decorator
